import java.util.*;
public class NittaD2 {
    static boolean long_mode = false;
    static boolean debug = false;
    public static void main(String[] args) {
	Scanner sc = new Scanner(System.in);
	for (int i=0; i<args.length; i++) {
	    if ("-l".equals(args[i])) {
		long_mode = true;
	    } else if ("-d".equals(args[i])) {
		debug = true;
	    }
	}
	while (true) {
	    int w = sc.nextInt();
	    int h = sc.nextInt();
	    if (w == 0 || h == 0) break;
	    TileMap tm = new TileMap(w,h);
	    for (int i=0; i<tm.getLength(); i++) tm.set(i,sc.nextInt());
	    int[] costs = new int[4];
	    for (int i=0; i<costs.length; i++) costs[i] = sc.nextInt();
	    FindCost fc = new FindCost(tm,costs);
	    System.out.println(fc.solve());
	}
    }
    static class FindCost {
	public static final int GoStraight = 0;
	public static final int RightFace = 1;
	public static final int AboutFace = 2;
	public static final int LeftFace = 3;
	public static final int HALT = 4;
	public static final int[][] dirs = {{0,1},{1,0},{0,-1},{-1,0}};	// (line,column)
	TileMap tm;
	int nTile;
	int nNode;
	Dijkstra dijkstra;
	public FindCost(TileMap tm,int[] costs) {
	    this.tm = tm;
	    nTile = tm.width * tm.height;
	    nNode = nTile * dirs.length; // Υ褿Ȥδθ4
	    if (debug) System.out.println("nTile="+nTile+", nNode="+nNode);
	    dijkstra = new Dijkstra();
	    for (int i=0; i<nNode; i++) {
		int tile = index2tile(i);
		int face = index2face(i);
		int line = tile2line(tile);
		int col = tile2column(tile);
		int val = tm.get(line,col);
		for (int rot=0; rot < dirs.length; rot++) {
		    int cost = (rot == val) ? 0 : costs[rot];
		    int dir = (face + rot) % dirs.length;
		    int nline = line + dirs[dir][0];
		    int ncol = col + dirs[dir][1];
		    if (tm.isOutOfRange(nline,ncol)) continue;
		    int ntile = linecolumn2tile(nline,ncol);
		    int nface = dir;
		    int nidx = tileface2index(ntile,nface);
		    dijkstra.addEdge(i,nidx,cost);
		    //matrix[i][nidx] = cost;
		    //if (debug) System.out.print(toString(matrix,i,nidx));
		}
	    }
	}
	int linecolumn2tile(int line,int col) { return tm.width * line + col; }
	int tileface2index(int tidx,int face) { return tidx * dirs.length + face; }
	int index2tile(int idx) { return idx / dirs.length; }
	int index2face(int idx) { return idx % dirs.length; }
	int tile2line(int tidx) { return tidx / tm.width; }
	int tile2column(int tidx) { return tidx % tm.width; }
	String index2string(int idx) {
	    int tile = index2tile(idx);
	    int face = index2face(idx);
	    int line = tile2line(tile);
	    int col = tile2column(tile);
	    return "("+line+","+col+"):"+face;
	}
	public int solve() {
	    int src=tileface2index(linecolumn2tile(0,0),0);
	    int dst1=tileface2index(linecolumn2tile(tm.height-1,tm.width-1),0);
	    int dst2=tileface2index(linecolumn2tile(tm.height-1,tm.width-1),1);
	    dijkstra.addDestination(dst1);
	    dijkstra.addDestination(dst2);
	    dijkstra.solve(src);
	    int len1 = dijkstra.getWeight(dst1);
	    int len2 = dijkstra.getWeight(dst2);
	    int mindst = (len1 < len2) ? dst1 : dst2;
	    if (long_mode) {
		int[] path=dijkstra.getPath(mindst);
		StringBuffer sb=new StringBuffer();
		for (int i=0; i<path.length; i++) {
		    sb.append(index2string(path[i]));
		    if (i!=path.length-1) sb.append(" -> ");
		}
		System.out.println(sb.toString());
	    }
	    doOutput(dijkstra,src,mindst);
	    return dijkstra.getWeight(mindst);
	}
	public String toString(int[][] m,int src,int dst) {
	    int val = m[src][dst];
	    return (val == Integer.MAX_VALUE) ? "":
		index2string(src) + "-("+val+")->" + index2string(dst)+"\n";
	}
	private void doOutput(Dijkstra dijkstra,int src,int dst) {
	    int nRoute = 0;
	    Vector<Node> nodes = dijkstra.nodes;
	    for (int i=0; i<nodes.size(); i++) {
		Vector<Edge> edges = nodes.get(i).edges;
		for (int j=0; j<edges.size(); j++) {
		    Edge e = edges.get(j);
		    if (e.src.id < e.dst.id) nRoute++;
		}
	    }
	    System.out.println(nodes.size()+" "+nRoute);
	    for (int i=0; i<nodes.size(); i++) {
		Vector<Edge> edges = nodes.get(i).edges;
		for (int j=0; j<edges.size(); j++) {
		    Edge e = edges.get(j);
		    int v = (e.weight == 0) ? 1 : (e.weight*10);
		    if (e.src.id < e.dst.id) 
			System.out.println(i+" "+j+" "+v);
		}
	    }
	    System.out.println(src+" "+dst);
	    System.exit(1);
	}
    }
    static class TileMap implements Cloneable {
	int width;
	int height;
	int[] map;
	static final int OUT_OF_RANGE = -1;
	public TileMap(int w,int h) {
	    width = w;
	    height = h;
	    map = new int[w*h];
	}
	public void clear() { clear(0); }
	public void clear(int val) { for (int i=0; i<map.length; i++) map[i] = val; }
	public int getLength() { return map.length; }
	public boolean isOutOfRange(int line,int col) {
	    return (line < 0 || line >= height || col < 0 || col >= width);
	}
	public boolean isOutOfRange(int idx) {
	    return (idx < 0 || idx >= map.length);
	}
	public int index2line(int idx) { return idx / width; }
	public int index2column(int idx) { return idx % width; }
	public int linecolumn2index(int line,int col) {return width*line+col;}
	public int get(int idx) {
	    return (isOutOfRange(idx)) ? OUT_OF_RANGE: map[idx];
	}
	public int get(int line,int col) {
	    return (isOutOfRange(line,col)) ? OUT_OF_RANGE
		: get(linecolumn2index(line,col));
	}
	public void set(int idx,int val) {
	    if (!isOutOfRange(idx)) map[idx] = val;
	}
	public void set(int line,int col,int val) {
	    if (!isOutOfRange(line,col)) set(linecolumn2index(line,col),val);
	}
	public String toString() {
	    StringBuffer sb = new StringBuffer();
	    for (int line=0; line<height; line++) {
		for (int col=0; col<width; col++) sb.append(get(line,col) + " ");
		sb.append("\n");
	    }
	    return sb.toString();
	}
    }
    static class Util {
	public static String toString(int[] a) {
	    StringBuffer sb = new StringBuffer();
	    sb.append("[");
	    if (a != null)
		for (int i=0; i<a.length; i++)
		    sb.append(((a[i]==Integer.MAX_VALUE)?"#":a[i])+" ");
	    sb.append("]");
	    return sb.toString();
	}
	public static String toString(int[][] a) {
	    StringBuffer sb = new StringBuffer();
	    sb.append("[");
	    if (a != null)
		for (int i=0; i<a.length; i++) sb.append(toString(a[i]) + "\n");
	    sb.append("]");
	    return sb.toString();
	}
    }
}
